@@ -2,7 +2,7 @@ package Test2;
 use strict;
 use warnings;
 
-our $VERSION = '1.302056';
+our $VERSION = '1.302103';
 
 
 1;
@@ -174,11 +174,11 @@ C<intercept()> and C<run_subtest()> are implemented.
 
 =head1 CONTACTING US
 
-Many Test2 developers and users lurk on L<irc://irc.perl.org/#perl>. We also
-have a slack team that can be joined by anyone with an C<@cpan.org> email
-address L<https://perl-test2.slack.com/> If you do not have an C<@cpan.org>
-email you can ask for a slack invite by emailing Chad Granum
-E<lt>exodist@cpan.orgE<gt>.
+Many Test2 developers and users lurk on L<irc://irc.perl.org/#perl-qa> and
+L<irc://irc.perl.org/#toolchain>. We also have a slack team that can be joined
+by anyone with an C<@cpan.org> email address L<https://perl-test2.slack.com/>
+If you do not have an C<@cpan.org> email you can ask for a slack invite by
+emailing Chad Granum E<lt>exodist@cpan.orgE<gt>.
 
 =head1 SOURCE
 
@@ -203,7 +203,7 @@ F<http://github.com/Test-More/test-more/>.
 
 =head1 COPYRIGHT
 
-Copyright 2016 Chad Granum E<lt>exodist@cpan.orgE<gt>.
+Copyright 2017 Chad Granum E<lt>exodist@cpan.orgE<gt>.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
